
function DEFINE_EXTENSION_axCheckBox()
{
    var definition_axCheckBox  =
    {
        /* Programmatic class name of this extension. It's not visible to end user */ 

        name   : "axCheckBox",
        group  : "vis",
        type   : "element",
        display: TEXT.dev("axCheckbox.Checkbox"),

        /* End-user visible name of this extension */

        properties:
        {
            onSetValue: { 
                type: "Event", 
                description: TEXT.dev("axCheckBox.onSetValue.description"), 
                defaultDynamic: "FIELD.getValue() == \"Y\"" 
            }, 
            onChecked: { 
                type: "Event", 
                description: TEXT.dev("axCheckBox.onChecked.description"), 
                defaultDynamic: "FIELD.setValue(\"Y\")" 
            },
            onUnchecked: {
                type: "Event", 
                description: TEXT.dev("axCheckBox.onUnchecked.description"), 
                defaultDynamic: "FIELD.setValue(\"N\")" 
            },
            onChange: { 
                type: "Event", 
                description: "Fires when this element changes state",
                defaultDynamic: ""
            },
            onFocus: {
                type: "Event", 
                description: "Fires when this element receives the focus", 
                defaultDynamic: "" 
            },
            onBlur: { 
                type: "Event", 
                description: "Fires when this element loses focus", 
                defaultDynamic: "" 
            }
        },

        /* Private properties of this object */

        _eCHECKBOX : null, 
        _oFIELD    : null,

        _chromeBlurred: false,

        init: function( field, fieldDiv )
        {
            this._oFIELD    = field;
            this._oDIV      = fieldDiv;
            
            var el = null;
            if (field.isInput) {
                el = document.createElement( "input" );
                el.type = "checkbox";
                $( el ).bind( "change", this, this._onChange );
                $( el ).bind( "focus", this, this._onFocus );
                $( el ).bind( "blur", this, this._onBlur );
                $( el ).bind( "click", this, this._onClickPrivate );                
                this.applyTabIndex( el );
            } else {
                // create text output element
                if (AXES.isTS2Engine) {
                    el = document.createElement("span");
                    el.className = "ax-terminal-textField";
                    if (field instanceof AXES.Terminal.Field.TextField) {
                        el.className += " ax_x" + field.attribute.substring(1).toUpperCase() + "o";
                    }
                } else {
                    el = this.createTextElement();
                }
            }
            
            this._eCHECKBOX = el;
            fieldDiv.appendChild( el );
            /* Listen for changes in the field value */
            field.addListener( "ValueChanged", this._onFieldValueChanged, this );
        },

        /* Render this control onto the screen */

        render: function(field, fieldDiv)
        {
            if (field.isInput) {
                this._eCHECKBOX.checked = this.getPropertyValue("onSetValue");
            } else {
                if ( this.getPropertyValue("onSetValue") ) {
                    $(this._eCHECKBOX).html("&#x2611;"); // decimal U+9745
                } else {
                    $(this._eCHECKBOX).html("&#x2610;"); // decimal U+9744
                }
            }
        },

        /* Clean up */

        destroy: function(field, fieldDiv)
        {
            var self = this;
            this._oFIELD    = null;
            this._oDIV      = null;
            this._eCHECKBOX = null;

            /* Remove any listeners */
            field.removeListener("ValueChanged", this._onFieldValueChanged);
        },

        /* Handles the onclick event of the check box */

        _onClickPrivate: function( e )
        {
            var self = e.data;

            if ( self.isDesignMode() )
                return;

            self._noFieldValueChangedEvent = true;
            self.raiseEvent( self._eCHECKBOX.checked ? "onChecked" : "onUnchecked", { FIELD: self._oFIELD } );
        },

        /* Handles the change in field value */
        _onFieldValueChanged: function( event )
        {
            if ( this._noFieldValueChangedEvent )
                delete this._noFieldValueChangedEvent;
            else
                this.render( this._oFIELD, this._oDIV );
            if ( AXES.isTS2Engine && $.browser.msie && "7.0" == $.browser.version )
                this.raiseEvent( "onChange", { FIELD: this._oFIELD } );
        },

        _onChange: function( e )
        {
            var self = e.data;

            if ( self.isDesignMode() )
                return;

            // simulate the element to receive focus on chrome and safari
            if ( $.browser.webkit || $.browser.safari ) this.focus();            
            self.raiseEvent( "onChange", { FIELD: self._oFIELD } );
        },

        _onFocus: function( e )
        {
            var self = e.data;
            if ( self.isDesignMode() )
                return;

            self.raiseEvent( "onFocus", { FIELD: self._oFIELD } );

            if ( $.browser.webkit || $.browser.safari )
                self._chromeBlurred = false;
        },

        _onBlur: function( e )
        {
            var self = e.data;
            if ( self.isDesignMode() )
                return;

            if ( $.browser.webkit || $.browser.safari )
            {
                if ( self._chromeBlurred )
                    self._chromeBlurred = false;
                else
                {
                    self.raiseEvent( "onBlur", { FIELD: self._oFIELD } );
                    self._chromeBlurred = true;
                }
            }
            else
                self.raiseEvent( "onBlur", { FIELD: self._oFIELD } );
        }
    };

    /* Register the extension with AXES */

    AXES.Extensions.add( definition_axCheckBox );
}

/* Self define this extension */

DEFINE_EXTENSION_axCheckBox ();

